#include <stdio.h>
#include <string.h>
#include <windows.h>

#include "settings.h"
#include "resource.h"
#include "guiutils.h"

#include "debug.h"

#include "connectdlg.h"

static void ConnectDlg_OnInitDialog(HWND hDlg, WPARAM wParam, LPARAM lParam)
{
	SetWindowLongPtr(hDlg, GWLP_USERDATA, lParam);
	struct Settings *lpSettings = (struct Settings*) lParam;

	CenterWindow(hDlg, GetParent(hDlg));

	const struct OptionList *list;
	list = GetPortNames();
	PopulateComboBoxString(GetDlgItem(hDlg, IDC_PORT), list, lpSettings->szPortName);

	list = GetBaudRateSettings();
	PopulateComboBoxDword(GetDlgItem(hDlg, IDC_BAUDRATE), list, lpSettings->dwBaudRate);

	list = GetDataBitsSettings();
	PopulateComboBoxDword(GetDlgItem(hDlg, IDC_DATABITS), list, lpSettings->bDataBits);

	list = GetParitySettings();
	PopulateComboBoxDword(GetDlgItem(hDlg, IDC_PARITY), list, lpSettings->bParity);

	list = GetStopBitsSettings();
	PopulateComboBoxDword(GetDlgItem(hDlg, IDC_STOPBITS), list, lpSettings->bStopBits);
}

static void ConnectDlg_OnOK(HWND hDlg, WPARAM wParam)
{
	struct Settings *lpSettings = (struct Settings*) GetWindowLongPtr(hDlg, GWLP_USERDATA);

	GetComboBoxSelectionString(GetDlgItem(hDlg, IDC_PORT), lpSettings->szPortName, sizeof(lpSettings->szPortName));

	DWORD dwValue;
	if (GetComboBoxSelectionDword(GetDlgItem(hDlg, IDC_BAUDRATE), &dwValue))
		lpSettings->dwBaudRate = dwValue;
	if (GetComboBoxSelectionDword(GetDlgItem(hDlg, IDC_DATABITS), &dwValue))
		lpSettings->bDataBits = (BYTE) dwValue;
	if (GetComboBoxSelectionDword(GetDlgItem(hDlg, IDC_PARITY), &dwValue))
		lpSettings->bParity = (BYTE) dwValue;
	if (GetComboBoxSelectionDword(GetDlgItem(hDlg, IDC_STOPBITS), &dwValue))
		lpSettings->bStopBits = (BYTE) dwValue;

	EndDialog(hDlg, IDOK);
}

static void ConnectDlg_OnCancel(HWND hDlg, WPARAM wParam)
{
	SendMessage(hDlg, WM_CLOSE, 0, 0);
}

static void ConnectDlg_OnClose(HWND hDlg)
{
	EndDialog(hDlg, IDCANCEL);
}

static const CTRL_HANDLER ConnectDlgProc_Handler[] = {
	{ IDOK,      ConnectDlg_OnOK     },
	{ IDCANCEL,  ConnectDlg_OnCancel },
};

INT_PTR CALLBACK ConnectDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	BOOL fResult = FALSE;

	switch (uMsg)
	{
	case WM_INITDIALOG:
		// [...] return TRUE to direct the system to set the keyboard focus [...]
		ConnectDlg_OnInitDialog(hDlg, wParam, lParam);
		fResult = TRUE;
		break;
	case WM_COMMAND:
		// If an application processes this message, it should return zero.
		HandleControlMessage(hDlg, wParam, ConnectDlgProc_Handler, ARRAYSIZE(ConnectDlgProc_Handler));
		break;
	case WM_CLOSE:
		// If an application processes this message, it should return zero.
		ConnectDlg_OnClose(hDlg);
		break;
	}

	return fResult;
}
