#include <stdio.h>
#include <string.h>
#include <windows.h>
#include <commdlg.h>

#include "settings.h"
#include "resource.h"
#include "guiutils.h"

#include "debug.h"

#include "pickfiledlg.h"

static void PickFileDlg_OnInitDialog(HWND hDlg, WPARAM wParam, LPARAM lParam)
{
	SetWindowLongPtr(hDlg, GWLP_USERDATA, lParam);
	CenterWindow(hDlg, GetParent(hDlg));

	struct PickFileDlgParam *lpData = (struct PickFileDlgParam*) lParam;

	if (lpData->bMode == PFD_MODE_SEND)
	{
		SetWindowText(hDlg, "Send file to target");
		SetWindowText(GetDlgItem(hDlg, IDC_LABEL_A), "Local path:");
		SetWindowText(GetDlgItem(hDlg, IDC_LABEL_B), "Remote path:");
		EnableWindow(GetDlgItem(hDlg, IDC_BUTTON_B), FALSE);
	}
	else
	{
		SetWindowText(hDlg, "Receive file from target");
		SetWindowText(GetDlgItem(hDlg, IDC_LABEL_A), "Remote path:");
		SetWindowText(GetDlgItem(hDlg, IDC_LABEL_B), "Local path:");
		EnableWindow(GetDlgItem(hDlg, IDC_BUTTON_A), FALSE);
	}
}

static void PickFileDlg_OnOK(HWND hDlg, WPARAM wParam)
{
	struct PickFileDlgParam *lpData = (struct PickFileDlgParam*) GetWindowLongPtr(hDlg, GWLP_USERDATA);

	if (lpData->bMode == PFD_MODE_SEND)
	{
		GetWindowText(GetDlgItem(hDlg, IDC_PATH_A), lpData->szLocalPath, sizeof(lpData->szLocalPath));
		GetWindowText(GetDlgItem(hDlg, IDC_PATH_B), lpData->szRemotePath, sizeof(lpData->szRemotePath));
	}
	else
	{
		GetWindowText(GetDlgItem(hDlg, IDC_PATH_A), lpData->szRemotePath, sizeof(lpData->szRemotePath));
		GetWindowText(GetDlgItem(hDlg, IDC_PATH_B), lpData->szLocalPath, sizeof(lpData->szLocalPath));
	}

	EndDialog(hDlg, IDOK);
}

static void PickFileDlg_OnButtonA(HWND hDlg, WPARAM wParam)
{
	// Send direction: A is local path, button is enabled, open for reading
	// Receive direction: A is remote path, button is disabled (and this function never called)

	OPENFILENAME ofn = { 0 };
	char szFile[MAX_PATH] = { 0 };
	ofn.lStructSize = sizeof(ofn);
	ofn.hwndOwner = hDlg;
	ofn.lpstrFile = szFile;
	ofn.nMaxFile = sizeof(szFile);
	ofn.lpstrFilter = "All (*.*)\0*.*\0";
	ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;

	if (GetOpenFileName(&ofn))
	{
		SetWindowText(GetDlgItem(hDlg, IDC_PATH_A), szFile);
	}
}

static void PickFileDlg_OnButtonB(HWND hDlg, WPARAM wParam)
{
	// Send direction: B is remote path, button is disabled (and this function never called)
	// Receive direction: B is local path, button is enabled, open for writing

	OPENFILENAME ofn = { 0 };
	char szFile[MAX_PATH] = { 0 };
	ofn.lStructSize = sizeof(ofn);
	ofn.hwndOwner = hDlg;
	ofn.lpstrFile = szFile;
	ofn.nMaxFile = sizeof(szFile);
	ofn.lpstrFilter = "All (*.*)\0*.*\0";
	ofn.Flags = OFN_OVERWRITEPROMPT;

	if (GetSaveFileName(&ofn))
	{
		SetWindowText(GetDlgItem(hDlg, IDC_PATH_B), szFile);
	}
}

static void PickFileDlg_OnCancel(HWND hDlg, WPARAM wParam)
{
	SendMessage(hDlg, WM_CLOSE, 0, 0);
}

static void PickFileDlg_OnClose(HWND hDlg)
{
	EndDialog(hDlg, IDCANCEL);
}

static const CTRL_HANDLER PickFileDlgProc_Handler[] = {
	{ IDC_BUTTON_A, PickFileDlg_OnButtonA },
	{ IDC_BUTTON_B, PickFileDlg_OnButtonB },
	{ IDOK,         PickFileDlg_OnOK      },
	{ IDCANCEL,     PickFileDlg_OnCancel  },
};

INT_PTR CALLBACK PickFileDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	BOOL fResult = FALSE;

	switch (uMsg)
	{
	case WM_INITDIALOG:
		// [...] return TRUE to direct the system to set the keyboard focus [...]
		PickFileDlg_OnInitDialog(hDlg, wParam, lParam);
		fResult = TRUE;
		break;
	case WM_COMMAND:
		// If an application processes this message, it should return zero.
		HandleControlMessage(hDlg, wParam, PickFileDlgProc_Handler, ARRAYSIZE(PickFileDlgProc_Handler));
		break;
	case WM_CLOSE:
		// If an application processes this message, it should return zero.
		PickFileDlg_OnClose(hDlg);
		break;
	}

	return fResult;
}
