#ifndef PROTODEF_H
#define PROTODEF_H

#include <windows.h>

#define MAX_TOOLBOX_SIZE     0xf000  // Maximum size of toolbox image
#define CMD_EXIT             0       // Function "exit with custom code"
#define CMD_GET_VERSION      1       // Function "get toolbox version"
#define CMD_GET_DRIVE_INFO   2       // Function "get drive information"
#define CMD_READ_SECTOR      3       // Function "read sector from drive"
#define CMD_WRITE_SECTOR     4       // Function "write sector to drive"
#define CMD_OPEN_FILE        5       // Function "open file"
#define CMD_GET_FILE_SIZE    6       // Function "get file size"
#define CMD_READ_FILE        7       // Function "read from file"
#define CMD_WRITE_FILE       8       // Function "write to file"
#define CMD_CLOSE_FILE       9       // Function "close file"
#define PRM_OPEN_FILE_READ   0       // Access mode "read"
#define PRM_OPEN_FILE_WRITE  1       // Access mode "write" (create)
#define RES_FLAG_SUCCESS     0x40    // Flag for successful requests
#define RES_FLAG_FAILURE     0x80    // Flag for failed requests

#define BIOS_CODE_SUCCESS    0x00    // BIOS status code indicating success
#define DOS_CODE_SUCCESS     0x00    // DOS status code indicating success
#define SECTOR_SIZE          512     // Size of disk sector in bytes
#define FILE_CHUNK_SIZE      512     // Maximum chunk size for file read/write
#define MAX_PATH_SIZE        260     // Maximum size of path on target


// Structs are used to create byte streams, no padding allowed!
#pragma pack(push, 1)

struct ExitRequest {
	BYTE bRequestCode;
	BYTE bExitCode;
};

struct ToolboxVersionRequest {
	BYTE bRequestCode;
};

struct ToolboxVersionResponse {
	BYTE bResponseCode;
	WORD wVersion;
};

struct GetDriveInfoRequest {
	BYTE bRequestCode;
	BYTE bDriveIndex;
};

struct GetDriveInfoResponse {
	BYTE bResponseCode;
	BYTE bBiosStatusCode;
	BYTE bDriveType;
	BYTE bNumberOfTracks;
	BYTE bNumberOfSides;
	BYTE bNumberOfSectors;
};

struct ReadSectorRequest {
	BYTE bRequestCode;
	BYTE bDriveIndex;
	BYTE bTrack;
	BYTE bSide;
	BYTE bSector;
};

struct ReadSectorResponse {
	BYTE bResponseCode;
	BYTE bBiosStatusCode;
	BYTE SectorData[SECTOR_SIZE];
};

struct WriteSectorRequest {
	BYTE bRequestCode;
	BYTE bDriveIndex;
	BYTE bTrack;
	BYTE bSide;
	BYTE bSector;
	BYTE SectorData[SECTOR_SIZE];
};

struct WriteSectorResponse {
	BYTE bResponseCode;
	BYTE bBiosStatusCode;
};

struct OpenFileRequest {
	BYTE bRequestCode;
	BYTE bAccessMode;
	BYTE szPath[MAX_PATH_SIZE];
};

struct OpenFileResponse {
	BYTE bResponseCode;
	BYTE bDosStatusCode;
	WORD wFileHandle;
};

struct GetFileSizeRequest {
	BYTE bRequestCode;
	WORD wFileHandle;
};

struct GetFileSizeResponse {
	BYTE bResponseCode;
	BYTE bDosStatusCode;
	DWORD dwFileSize;
};

struct ReadFileRequest {
	BYTE bRequestCode;
	WORD wFileHandle;
};

struct ReadFileResponse {
	BYTE bResponseCode;
	BYTE bDosStatusCode;
	WORD wBytesRead;
	BYTE FileData[FILE_CHUNK_SIZE];
};

struct WriteFileRequest {
	BYTE bRequestCode;
	WORD wFileHandle;
	WORD wBytesToWrite;
	BYTE FileData[FILE_CHUNK_SIZE];
};

struct WriteFileResponse {
	BYTE bResponseCode;
	BYTE bDosStatusCode;
	WORD wBytesWritten;
};

struct CloseFileRequest {
	BYTE bRequestCode;
	WORD wFileHandle;
};

struct CloseFileResponse {
	BYTE bResponseCode;
	BYTE bDosStatusCode;
};

#pragma pack(pop)

#endif
