#include <windows.h>

#include "bgjob.h"
#include "resource.h"
#include "guiutils.h"

#include "transferdlg.h"

static void TransferDlg_OnInitDialog(HWND hDlg, WPARAM wParam, LPARAM lParam)
{
	SetWindowLongPtr(hDlg, GWLP_USERDATA, lParam);
	struct TransferDlgData *lpData = (struct TransferDlgData*) lParam;

	char szBuffer[MAX_PATH];
	TruncateText(GetDlgItem(hDlg, IDC_SOURCE), lpData->lpSource, szBuffer, sizeof(szBuffer), DT_PATH_ELLIPSIS);
	SetWindowText(GetDlgItem(hDlg, IDC_SOURCE), szBuffer);
	TruncateText(GetDlgItem(hDlg, IDC_DESTINATION), lpData->lpDestination, szBuffer, sizeof(szBuffer), DT_PATH_ELLIPSIS);
	SetWindowText(GetDlgItem(hDlg, IDC_DESTINATION), szBuffer);

	lpData->dwBeginOfTransfer = GetTickCount();
	lpData->hProgressBar = CreateProgressBar(hDlg, GetDlgItem(hDlg, IDC_PROGRESS_DUMMY));
	lpData->lpUpdateProgressProc(hDlg, lpData);

	SetBackgroundJobOwner(lpData->lpJob, hDlg);
	StartBackgroundJob(lpData->lpJob);
}

static void TransferDlg_OnJobProgress(HWND hDlg)
{
	struct TransferDlgData *lpData = (struct TransferDlgData*) GetWindowLongPtr(hDlg, GWLP_USERDATA);
	lpData->lpUpdateProgressProc(hDlg, lpData);
}

static void TransferDlg_OnCancel(HWND hDlg, WPARAM wParam)
{
	SendMessage(hDlg, WM_CLOSE, 0, 0);
}

static void TransferDlg_OnClose(HWND hDlg)
{
	struct TransferDlgData *lpData = (struct TransferDlgData*) GetWindowLongPtr(hDlg, GWLP_USERDATA);
	CancelBackgroundJob(lpData->lpJob);

	// Note: The system expects that WM_CLOSE is handled by a call to EndDialog.
	//       Failing to do so causes the system to post WM_CLOSE messages again and again.
	//       On the other hand the dialog result should be the job result, so it cannot be
	//       ended before the job result code is known.
	//       By waiting here, the main thread is blocked until the job thread has posted
	//       the WM_JOB_FINISHED message, which triggers a call to EndDialog as well,
	//       with the result code to be passed to the dialog owner.
	WaitForBackgroundJob(lpData->lpJob, INFINITE);
}

static void TransferDlg_OnJobFinished(HWND hDlg, WORD wResult)
{
	EndDialog(hDlg, wResult);
}

static void TransferDlg_OnDestroy(HWND hDlg)
{
	struct TransferDlgData *lpData = (struct TransferDlgData*) GetWindowLongPtr(hDlg, GWLP_USERDATA);

	if (lpData->hProgressBar != NULL)
	{
		CloseWindow(lpData->hProgressBar);
		lpData->hProgressBar = NULL;
	}
}

static const CTRL_HANDLER TransferDlgProc_Handler[] = {
	{ IDCANCEL, TransferDlg_OnCancel },
};

INT_PTR CALLBACK TransferDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	BOOL fResult = FALSE;

	switch (uMsg)
	{
	case WM_INITDIALOG:
		// [...] return TRUE to direct the system to set the keyboard focus [...]
		TransferDlg_OnInitDialog(hDlg, wParam, lParam);
		fResult = TRUE;
		break;
	case WM_COMMAND:
		// If an application processes this message, it should return zero.
		HandleControlMessage(hDlg, wParam, TransferDlgProc_Handler, ARRAYSIZE(TransferDlgProc_Handler));
		break;
	case WM_JOB_PROGRESS:
		TransferDlg_OnJobProgress(hDlg);
		break;
	case WM_JOB_FINISHED:
		TransferDlg_OnJobFinished(hDlg, (WORD) wParam);
		break;
	case WM_CLOSE:
		// If an application processes this message, it should return zero.
		TransferDlg_OnClose(hDlg);
		break;
	case WM_DESTROY:
		TransferDlg_OnDestroy(hDlg);
		break;
	}

	return fResult;
}
