#include <stdio.h>
#include <stdlib.h>
#include <windows.h>

#include "settings.h"
#include "resource.h"
#include "guiutils.h"

#include "debug.h"

#include "tweaksdlg.h"

#define MAX_VALUE 50

static void UpdateControls(HWND hDlg, const struct Settings *lpSettings)
{
	char szValue[MAX_VALUE];
	sprintf_s(szValue, sizeof(szValue), "%i", lpSettings->wDelayBeforeWrite);
	SetWindowText(GetDlgItem(hDlg, IDC_DELAY_BEFORE_WRITE), szValue);
	sprintf_s(szValue, sizeof(szValue), "%i", lpSettings->wDelayAfterWrite);
	SetWindowText(GetDlgItem(hDlg, IDC_DELAY_AFTER_WRITE), szValue);
	sprintf_s(szValue, sizeof(szValue), "%i", lpSettings->wDelayBeforeRead);
	SetWindowText(GetDlgItem(hDlg, IDC_DELAY_BEFORE_READ), szValue);
	sprintf_s(szValue, sizeof(szValue), "%i", lpSettings->wDelayAfterRead);
	SetWindowText(GetDlgItem(hDlg, IDC_DELAY_AFTER_READ), szValue);
	sprintf_s(szValue, sizeof(szValue), "%i", lpSettings->wDelayBeforeReadAfterWrite);
	SetWindowText(GetDlgItem(hDlg, IDC_DELAY_BEFORE_READ_AFTER_WRITE), szValue);
	sprintf_s(szValue, sizeof(szValue), "%i", lpSettings->wDelayBeforeWriteAfterRead);
	SetWindowText(GetDlgItem(hDlg, IDC_DELAY_BEFORE_WRITE_AFTER_READ), szValue);
	sprintf_s(szValue, sizeof(szValue), "%i", lpSettings->wReadTimeout);
	SetWindowText(GetDlgItem(hDlg, IDC_READ_TIMEOUT), szValue);

	CheckDlgButton(hDlg, IDC_OUTX_CTS_FLOW, lpSettings->fOutxCtsFlow ? BST_CHECKED : BST_UNCHECKED);
	CheckDlgButton(hDlg, IDC_OUTX_DSR_FLOW, lpSettings->fOutxDsrFlow ? BST_CHECKED : BST_UNCHECKED);

	const struct OptionList *list;
	list = GetDtrControlSettings();
	PopulateComboBoxDword(GetDlgItem(hDlg, IDC_DTR_CONTROL), list, lpSettings->bDtrControl);
	list = GetRtsControlSettings();
	PopulateComboBoxDword(GetDlgItem(hDlg, IDC_RTS_CONTROL), list, lpSettings->bRtsControl);
}

static void TweaksDlg_OnInitDialog(HWND hDlg, WPARAM wParam, LPARAM lParam)
{
	SetWindowLongPtr(hDlg, GWLP_USERDATA, lParam);
	const struct Settings *lpSettings = (struct Settings*) lParam;

	CenterWindow(hDlg, GetParent(hDlg));
	UpdateControls(hDlg, lpSettings);
}

static void TweaksDlg_OnOK(HWND hDlg, WPARAM wParam)
{
	struct Settings *lpSettings = (struct Settings*) GetWindowLongPtr(hDlg, GWLP_USERDATA);

	DWORD dwValue;
	char szValue[MAX_VALUE];

	GetWindowText(GetDlgItem(hDlg, IDC_DELAY_BEFORE_WRITE), szValue, sizeof(szValue));
	lpSettings->wDelayBeforeWrite = (WORD) strtoul(szValue, NULL, 0);
	GetWindowText(GetDlgItem(hDlg, IDC_DELAY_AFTER_WRITE), szValue, sizeof(szValue));
	lpSettings->wDelayAfterWrite = (WORD) strtoul(szValue, NULL, 0);
	GetWindowText(GetDlgItem(hDlg, IDC_DELAY_BEFORE_READ), szValue, sizeof(szValue));
	lpSettings->wDelayBeforeRead = (WORD) strtoul(szValue, NULL, 0);
	GetWindowText(GetDlgItem(hDlg, IDC_DELAY_AFTER_READ), szValue, sizeof(szValue));
	lpSettings->wDelayAfterRead = (WORD) strtoul(szValue, NULL, 0);
	GetWindowText(GetDlgItem(hDlg, IDC_DELAY_BEFORE_READ_AFTER_WRITE), szValue, sizeof(szValue));
	lpSettings->wDelayBeforeReadAfterWrite = (WORD) strtoul(szValue, NULL, 0);
	GetWindowText(GetDlgItem(hDlg, IDC_DELAY_BEFORE_WRITE_AFTER_READ), szValue, sizeof(szValue));
	lpSettings->wDelayBeforeWriteAfterRead = (WORD) strtoul(szValue, NULL, 0);
	GetWindowText(GetDlgItem(hDlg, IDC_READ_TIMEOUT), szValue, sizeof(szValue));
	lpSettings->wReadTimeout = (WORD) strtoul(szValue, NULL, 0);
	lpSettings->fOutxCtsFlow = IsDlgButtonChecked(hDlg, IDC_OUTX_CTS_FLOW) == BST_CHECKED;
	lpSettings->fOutxDsrFlow = IsDlgButtonChecked(hDlg, IDC_OUTX_DSR_FLOW) == BST_CHECKED;
	if (GetComboBoxSelectionDword(GetDlgItem(hDlg, IDC_DTR_CONTROL), &dwValue))
		lpSettings->bDtrControl = (BYTE) dwValue;
	if (GetComboBoxSelectionDword(GetDlgItem(hDlg, IDC_RTS_CONTROL), &dwValue))
		lpSettings->bRtsControl = (BYTE) dwValue;

	EndDialog(hDlg, IDOK);
}

static void TweaksDlg_OnLoadDefaults(HWND hDlg, WPARAM wParam)
{
	struct Settings defaults;
	LoadTweaksDefaults(&defaults);
	UpdateControls(hDlg, &defaults);
}

static void TweaksDlg_OnCancel(HWND hDlg, WPARAM wParam)
{
	SendMessage(hDlg, WM_CLOSE, 0, 0);
}

static void TweaksDlg_OnClose(HWND hDlg)
{
	EndDialog(hDlg, IDCANCEL);
}

static const CTRL_HANDLER TweaksDlgProc_Handler[] = {
	{ IDOK,               TweaksDlg_OnOK     },
	{ IDC_LOAD_DEFAULTS,  TweaksDlg_OnLoadDefaults },
	{ IDCANCEL,           TweaksDlg_OnCancel },
};

INT_PTR CALLBACK TweaksDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	BOOL fResult = FALSE;

	switch (uMsg)
	{
	case WM_INITDIALOG:
		// [...] return TRUE to direct the system to set the keyboard focus [...]
		TweaksDlg_OnInitDialog(hDlg, wParam, lParam);
		fResult = TRUE;
		break;
	case WM_COMMAND:
		// If an application processes this message, it should return zero.
		HandleControlMessage(hDlg, wParam, TweaksDlgProc_Handler, ARRAYSIZE(TweaksDlgProc_Handler));
		break;
	case WM_CLOSE:
		// If an application processes this message, it should return zero.
		TweaksDlg_OnClose(hDlg);
		break;
	}

	return fResult;
}
