#!/usr/bin/perl

use strict;
my $width = 79;
my $disasm = 'ndisasm';

sub printHeader
{
    my $path = $_[0];
    my $size = $_[1];
    my $mtime = $_[2];

    my $length = 0;
    $length = length $path if (length $path > $length);
    $length = length $size if (length $size > $length);
    $length = length $mtime if (length $mtime > $length);

    $length = $width - 21 if ($length + 21 < $width);

    print "###################"          . "#" x  $length                  . "##\n";
    print "#                  "          . " " x  $length                  . " #\n";
    print "# Filename:        " . $path  . " " x ($length - length $path)  . " #\n";
    print "#                  "          . " " x  $length                  . " #\n";
    print "# Size:            " . $size  . " " x ($length - length $size)  . " #\n";
    print "#                  "          . " " x  $length                  . " #\n";
    print "# Last modified:   " . $mtime . " " x ($length - length $mtime) . " #\n";
    print "#                  "          . " " x  $length                  . " #\n";
    print "###################"          . "#" x  $length                  . "##\n";
}

sub printListing
{
    my @listing = @{$_[0]};

    my $inslen = 0;
    my $maxbytes = 0;
    foreach (@listing)
    {
        if (m/^[0-9A-F]+\s+([0-9A-F]+)\s+([^\r\n]+)/)
        {
            $inslen = length $2 if (length $2 > $inslen);
            $maxbytes = length $1 / 2 if (length $1 /2 > $maxbytes);
        }
    }

    my $inscap = "Instruction";
    $inslen = length $inscap if (length $inscap > $inslen);

    print "+------+-" . "-" x $inslen . "-+----" . "---" x ($maxbytes - 1) . "+-----" . "----" x ($maxbytes - 1) . "+\n";
    print "| Addr | $inscap" . " " x ($inslen - length $inscap) . " | Hex" . "   " x ($maxbytes - 1) . "| Dec " . "    " x ($maxbytes - 1) . "|\n";
    print "+------+-" . "-" x $inslen . "-+----" . "---" x ($maxbytes - 1) . "+-----" . "----" x ($maxbytes - 1) . "+\n";

    my $addr = 0;
    foreach (@listing)
    {
        if (m/^[0-9A-F]+\s+([0-9A-F]+)\s+([^\r\n]+)/)
        {
            printf("| %.4X | ", $addr);
            print $2 . " " x ($inslen - length $2) . " | ";

            my $bytes = length($1) / 2;

            for (my $i = 0; $i < $bytes; $i++)
            {
                my $h = substr $1, ($i * 2), 2;
                print "$h ";
            }
            print "   " x ($maxbytes - $bytes) . "| ";

            for (my $i = 0; $i < $bytes; $i++)
            {
                my $h = substr $1, ($i * 2), 2;
                my $d = hex $h;
                my $fill = 3 - length $d;
                print " " x $fill . "$d ";
            }
            print "    " x ($maxbytes - $bytes) . "|\n";

            $addr += $bytes;
        }
    }

    print "+------+-" . "-" x $inslen . "-+----" . "---" x ($maxbytes - 1) . "+-----" . "----" x ($maxbytes - 1) . "+\n";
}

sub main
{
    if (scalar @ARGV != 1)
    {
        print "Usage: listing.pl <binary file>\n";
        exit 0;
    }

    my $path = $ARGV[0];
    die "File doesn't exist" if not -e $path;

    my $size = (stat($path))[7];
    my $mtime = (stat($path))[9];

    printHeader($path, "$size bytes", scalar localtime($mtime));
    print "\n";

    my $cmd = "$disasm -o100h $path";
    my @listing = qx($cmd);
    printListing(\@listing);
}

main();
