#!/usr/bin/perl

use strict;

my $name = "pc-serial-loader";
my $ret;

my @files = (
    { name => "readme.txt",         srcdir => "doc",                dstdir => "."      },
    { name => "LICENSE",            srcdir => ".",                  dstdir => "."      },
    { name => "initld.com",         srcdir => "src/initld",         dstdir => "target" },
    { name => "toolbox.bin",        srcdir => "src/toolbox",        dstdir => "target" },
    { name => "toolbox-client.exe", srcdir => "src/toolbox-client", dstdir => "client" },
);

if (scalar @ARGV != 1)
{
    print "usage: makebindist.pl <version name>\n";
    exit 1;
}

print "Create binary distribution for version $ARGV[0]\n";

my $distdir = "$name-bin-$ARGV[0]";
die "Directory \"$distdir\" already exists" if (-e $distdir);
print "* CREATE   $distdir\n";
mkdir $distdir or die "Error creating directory";

foreach (@files)
{
    my %entry = %{$_};

    my $dir = "$distdir/$entry{dstdir}";
    my $src = "$entry{srcdir}/$entry{name}";
    my $dst = "$dir/$entry{name}";

    if (not -e $dir)
    {
        mkdir $dir or die "couldn't create dir";
    }

    print "* COPY     $src => $dst\n";
    $ret = system("cp -a $src $dst");
    die "Error copying file" if ($ret != 0);
}

my $initld = "$distdir/target/initld.com";
my $list = "$distdir/target/initld.txt";
print "* LISTING  $initld => $list\n";
$ret = system("tools/listing.pl $initld > $list");
die "Error creating listing" if ($ret != 0);

my $archive = "$distdir.zip";
print "* ARCHIVE  $archive\n";
$ret = system("$ENV{'ZIP'} a -r -bso0 $archive $distdir");
die "Error creating archive" if ($ret != 0);

print "* REMOVE   $distdir\n";
$ret = system("rm -r $distdir");
die "Error removing directory" if ($ret != 0);

print "Finished successfully.\n";
